@@ -20,9 +20,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <string.h>
 #include <stdlib.h>
-#include <popt.h>
 #include <sys/time.h>
 #include <gdk/gdkwindow.h>
 #include <gdk/gdkx.h>
@@ -31,6 +31,7 @@
 #include "magnifier.h"
 #include "magnifier-private.h"
 #include "zoom-region.h"
+#include "gmag-events.h"
 #include "GNOME_Magnifier.h"
 
 #define ENV_STRING_MAX_SIZE 128
@@ -38,98 +39,109 @@
 GNOME_Magnifier_ZoomRegion zoom_region;
 
 typedef struct {
-	gchar *target_display;
-	gchar *source_display;
-	gchar *cursor_set;
-	gchar *smoothing_type;
-	float zoom_factor;
-	float zoom_factor_x;
-	float zoom_factor_y;
-	int   refresh_time;
-	int   mouse_poll_time;
-	int   cursor_size;
-	float cursor_scale_factor;
-	guint32 cursor_color;
-	int   vertical_split;
-	int   horizontal_split;
-	int   fullscreen;
-	int   mouse_follow;
-	int   invert_image;
-	int   no_initial_region;
-	int   timing_iterations;
-	int   timing_output;
-	int   timing_delta_x;
-	int   timing_delta_y;
-	int   timing_pan_rate;
-	int   smooth_scroll;
-	int   border_width;
-	unsigned long  border_color;
-	int   test_pattern;
-        int   is_override_redirect;
-        int   ignore_damage;
+	gchar    *target_display;
+	gchar    *source_display;
+	gchar    *cursor_set;
+	gchar    *smoothing_type;
+	gdouble   zoom_factor;
+	gdouble   zoom_factor_x;
+	gdouble   zoom_factor_y;
+	gint      refresh_time;
+	gint      mouse_poll_time;
+	gint      cursor_size;
+	gdouble   cursor_scale_factor;
+	gint64    cursor_color;
+	gboolean  vertical_split;
+	gboolean  horizontal_split;
+	gboolean  fullscreen;
+	gboolean  mouse_follow;
+	gboolean  invert_image;
+	gboolean  no_initial_region;
+	gint      timing_iterations;
+	gboolean  timing_output;
+	gint      timing_delta_x;
+	gint      timing_delta_y;
+	gint      timing_pan_rate;
+	gboolean  smooth_scroll;
+	gint      border_width;
+	gint64    border_color;
+	gboolean  test_pattern;
+	gboolean  is_override_redirect;
+	gboolean  ignore_damage;
+#ifdef HAVE_COMPOSITE
+	gboolean   ignore_composite;
+#endif /* HAVE_COMPOSITE */
+	gboolean  print_version;
 } MagnifierOptions;
 
-static MagnifierOptions global_options = { NULL,
-					   NULL,
-					   "default",
-					   "none",
-					   2.0,
-					   0.0,
-					   0.0,
-					   500,
-					   50,
-					   0,
-					   0.0F,
-					   0xFF000000,
-					   0,
-					   0,
-					   0,
-					   0,
-					   0,	
-                                   	   0,
-					   0,
-					   0,
-					   10,
-					   10,
-					   0,
-					   1,
-					   0,
-					   0,
-					   0,
-					   0,
-					   0
+static MagnifierOptions global_options = { NULL,      /* target_display */
+					   NULL,      /* source_display */
+					   "default", /* cursor_set */
+					   "none",    /* smoothing_type */
+					   2.0,       /* zoom_factor */
+					   0.0,       /* zoom_factor_x */
+					   0.0,       /* zoom_factor_y */
+					   500,       /* refresh_time */
+					   50,        /* mouse_poll_time */
+					   0,         /* cursor_size */
+					   0.0F,      /* cursor_scale_factor */
+					   0xFF000000,/* cursor_color */
+					   0,         /* vertical_split */
+					   0,         /* horizontal_split */
+					   0,         /* fullscreen */
+					   0,         /* mouse_follow */
+					   0,	      /* invert_image */
+                                   	   0,         /* no_initial_region */
+					   0,         /* timing_iterations */
+					   0,         /* timing_output */
+					   10,        /* timing_delta_x */
+					   10,        /* timing_delat_y */
+					   0,         /* timing_pan_rate */
+					   1,         /* smooth_scroll */
+					   0,         /* border_width */
+					   0,         /* border_color */
+					   0,         /* test_pattern */
+					   0,         /* is_override_redirect*/
+					   0          /* ignore_damage */
+#ifdef HAVE_COMPOSITE
+					   ,0         /* ignore_composite */
+#endif /* HAVE_COMPOSITE */
+					   ,0         /* print_version */
                                          };
 
-struct poptOption magnifier_options [] = {
-	{"target-display", 't', POPT_ARG_STRING, &global_options.target_display, 't', "specify display on which to show magnified view", NULL},
-	{"source-display", 's', POPT_ARG_STRING, &global_options.source_display, 's', "specify display to magnify", NULL},
-	{"cursor-set", '\0', POPT_ARG_STRING, &global_options.cursor_set, '\0', "cursor set to use in target display", NULL},
-	{"cursor-size", '\0', POPT_ARG_INT, &global_options.cursor_size, '\0', "cursor size to use (overrides cursor-scale-factor)", NULL},
-	{"cursor-scale-factor", '\0', POPT_ARG_FLOAT, &global_options.cursor_scale_factor, '\0', "cursor scale factor", NULL},
-	{"cursor-color", '\0', POPT_ARG_LONG, &global_options.cursor_color, '\0', "cursor color (applied to \'black\' pixels", NULL},
-	{"vertical", 'v', POPT_ARG_NONE, &global_options.vertical_split, 'v', "split screen vertically (if target display = source display)", NULL},
-	{"horizontal", 'h', POPT_ARG_NONE, &global_options.horizontal_split, 'h', "split screen horizontally (if target display = source display)", NULL},
-	{"mouse-follow", 'm', POPT_ARG_NONE, &global_options.mouse_follow, 'm', "track mouse movements", NULL},
-	{"refresh-time", 'r', POPT_ARG_INT, &global_options.refresh_time, 'r', "minimum refresh time for idle, in ms", NULL},
-	{"mouse-latency", '\0', POPT_ARG_INT, &global_options.mouse_poll_time, '\0', "maximum mouse latency time, in ms", NULL},
-	{"zoom-factor", 'z', POPT_ARG_FLOAT, &global_options.zoom_factor, 'z', "zoom (scale) factor used to magnify source display", NULL}, 
-	{"invert-image", 'i', POPT_ARG_NONE, &global_options.invert_image, 'i', "invert the image colormap", NULL}, 
-	{"no-initial-region", '\0', POPT_ARG_NONE, &global_options.no_initial_region, '\0', "don't create an initial zoom region", NULL},
-	{"timing-iterations", '\0', POPT_ARG_INT, &global_options.timing_iterations, '\0', "iterations to run timing benchmark test (0=continuous)", NULL},
-	{"timing-output", '\0', POPT_ARG_NONE, &global_options.timing_output, '\0', "display performance ouput", NULL},
-	{"timing-pan-rate", '\0', POPT_ARG_INT, &global_options.timing_pan_rate, '\0', "timing pan rate in lines per frame", NULL},
-	{"timing-delta-x", '\0', POPT_ARG_INT, &global_options.timing_delta_x, '\0', "pixels to pan in x-dimension each frame in timing update test", NULL},
-	{"timing-delta-y", '\0', POPT_ARG_INT, &global_options.timing_delta_y, '\0', "pixels to pan in y-dimension each frame in timing update test", NULL},
-	{"smoothing-type", '\0', POPT_ARG_STRING, &global_options.smoothing_type, '\0', "image smoothing algorithm to apply (bilinear-interpolation | none)", NULL},
-	{"fullscreen", 'f', POPT_ARG_NONE, &global_options.fullscreen, '\0', "fullscreen magnification, covers entire target display [REQUIRES --source-display and --target-display]", NULL},
-	{"smooth-scrolling", '\0', POPT_ARG_NONE, &global_options.smooth_scroll, '\0', "use smooth scrolling", NULL},
-	{"border-size", 'b', POPT_ARG_INT, &global_options.border_width, '\0', "width of border", NULL},
-	{"border-color", 'c', POPT_ARG_LONG, &global_options.border_color, '\0', "border color specified as (A)RGB 23-bit value, Alpha-MSB", NULL},
-	{"use-test-pattern", '\0', POPT_ARG_NONE, &global_options.test_pattern, '\0', "use test pattern as source", NULL},
-	{"override-redirect", '\0', POPT_ARG_NONE, &global_options.is_override_redirect, '\0', "make the magnifier window totally unmanaged by the window manager", NULL},
-	{"ignore-damage", '\0', POPT_ARG_NONE, &global_options.ignore_damage, '\0', "ignore the X server DAMAGE extension, if present", NULL},
-	POPT_AUTOHELP
-	{NULL, 0, 0, NULL, 0, 0}
+static GOptionEntry magnifier_options [] = {
+	{"target-display", 't', 0, G_OPTION_ARG_STRING, &global_options.target_display, "specify display on which to show magnified view", NULL},
+	{"source-display", 's', 0, G_OPTION_ARG_STRING, &global_options.source_display, "specify display to magnify", NULL},
+	{"cursor-set", 0, 0, G_OPTION_ARG_STRING, &global_options.cursor_set, "cursor set to use in target display", NULL},
+	{"cursor-size", 0, 0, G_OPTION_ARG_INT, &global_options.cursor_size, "cursor size to use (overrides cursor-scale-factor)", NULL},
+	{"cursor-scale-factor", 0, 0, G_OPTION_ARG_DOUBLE, &global_options.cursor_scale_factor, "cursor scale factor", NULL},
+	{"cursor-color", 0, 0, G_OPTION_ARG_INT64, &global_options.cursor_color, "cursor color (applied to \'black\' pixels)", NULL},
+	{"vertical", 'v', 0, G_OPTION_ARG_NONE, &global_options.vertical_split, "split screen vertically (if target display = source display)", NULL},
+	{"horizontal", 'h', 0, G_OPTION_ARG_NONE, &global_options.horizontal_split, "split screen horizontally (if target display = source display)", NULL},
+	{"mouse-follow", 'm', 0, G_OPTION_ARG_NONE, &global_options.mouse_follow, "track mouse movements", NULL},
+	{"refresh-time", 'r', 0, G_OPTION_ARG_INT, &global_options.refresh_time, "minimum refresh time for idle, in ms", NULL},
+	{"mouse-latency", 0, 0, G_OPTION_ARG_INT, &global_options.mouse_poll_time, "maximum mouse latency time, in ms", NULL},
+	{"zoom-factor", 'z', 0, G_OPTION_ARG_DOUBLE, &global_options.zoom_factor, "zoom (scale) factor used to magnify source display", NULL}, 
+	{"invert-image", 'i', 0, G_OPTION_ARG_NONE, &global_options.invert_image, "invert the image colormap", NULL}, 
+	{"no-initial-region", 0, 0, G_OPTION_ARG_NONE, &global_options.no_initial_region, "don't create an initial zoom region", NULL},
+	{"timing-iterations", 0, 0, G_OPTION_ARG_INT, &global_options.timing_iterations, "iterations to run timing benchmark test (0=continuous)", NULL},
+	{"timing-output", 0, 0, G_OPTION_ARG_NONE, &global_options.timing_output, "display performance ouput", NULL},
+	{"timing-pan-rate", 0, 0, G_OPTION_ARG_INT, &global_options.timing_pan_rate, "timing pan rate in lines per frame", NULL},
+	{"timing-delta-x", 0, 0, G_OPTION_ARG_INT, &global_options.timing_delta_x, "pixels to pan in x-dimension each frame in timing update test", NULL},
+	{"timing-delta-y", 0, 0, G_OPTION_ARG_INT, &global_options.timing_delta_y, "pixels to pan in y-dimension each frame in timing update test", NULL},
+	{"smoothing-type", 0, 0, G_OPTION_ARG_STRING, &global_options.smoothing_type, "image smoothing algorithm to apply (bilinear-interpolation | none)", NULL},
+	{"fullscreen", 'f', 0, G_OPTION_ARG_NONE, &global_options.fullscreen, "fullscreen magnification, covers entire target display [REQUIRES --source-display and --target-display]", NULL},
+	{"smooth-scrolling", 0, 0, G_OPTION_ARG_NONE, &global_options.smooth_scroll, "use smooth scrolling", NULL},
+	{"border-size", 'b', 0, G_OPTION_ARG_INT, &global_options.border_width, "width of border", NULL},
+	{"border-color", 'c', 0, G_OPTION_ARG_INT64, &global_options.border_color, "border color specified as (A)RGB 23-bit value, Alpha-MSB", NULL},
+	{"use-test-pattern", 0, 0, G_OPTION_ARG_NONE, &global_options.test_pattern, "use test pattern as source", NULL},
+	{"override-redirect", 0, 0, G_OPTION_ARG_NONE, &global_options.is_override_redirect, "make the magnifier window totally unmanaged by the window manager", NULL},
+	{"ignore-damage", 0, 0, G_OPTION_ARG_NONE, &global_options.ignore_damage, "ignore the X server DAMAGE extension, if present", NULL},
+#ifdef HAVE_COMPOSITE
+	{"ignore-composite", 0, 0, G_OPTION_ARG_NONE, &global_options.ignore_composite, "ignore the X server COMPOSITE extension, if present", NULL},
+#endif /* HAVE_COMPOSITE */
+	{"version", 0, 0, G_OPTION_ARG_NONE, &global_options.print_version, "print version", NULL},
+	{NULL}
 };
 
 static void
@@ -343,7 +355,7 @@ magnifier_main_refresh_all (gpointer data)
 int
 main (int argc, char** argv)
 {
-  poptContext ctx;
+  GOptionContext *context;
   GNOME_Magnifier_RectBounds *roi = GNOME_Magnifier_RectBounds__alloc();
   GNOME_Magnifier_RectBounds *viewport = GNOME_Magnifier_RectBounds__alloc();
   CORBA_any *viewport_any;
@@ -359,16 +371,17 @@ main (int argc, char** argv)
   }
   CORBA_exception_init (&ev);
   
-  ctx = poptGetContext ("magnifier",
-			argc,
-			(const char **)argv,
-			magnifier_options,
-			0);
+  context = g_option_context_new ("- a screen magnifier for Gnome");
+  g_option_context_set_description (context, "Report bugs to http://bugzilla.gnome.org\n");
+  g_option_context_add_main_entries (context, magnifier_options, "main options");
+  g_option_context_set_ignore_unknown_options (context, TRUE);
+  g_option_context_parse(context, &argc, &argv, NULL);
+  g_option_context_free(context);
 
-  while (poptGetNextOpt (ctx) >= 0)
-	/**/;
-
-  poptFreeContext (ctx);
+  if (global_options.print_version) {
+	  g_print ("%s\n", VERSION);
+	  return 0;
+  }
 
   /** 
    * slight hack, we set $DISPLAY if --target-display is set.
@@ -403,6 +416,11 @@ main (int argc, char** argv)
   {
       g_setenv ("MAGNIFIER_IGNORE_DAMAGE", "1", TRUE);
   }
+#ifdef HAVE_COMPOSITE
+  if (global_options.ignore_composite) {
+	  g_setenv ("MAGNIFIER_IGNORE_COMPOSITE", "1", TRUE);
+  }
+#endif /* HAVE_COMPOSITE */
 
   magnifier = magnifier_new (global_options.is_override_redirect);
   
@@ -451,6 +469,7 @@ main (int argc, char** argv)
   }
   if (global_options.horizontal_split) {
 	  screen_height /= 2;
+	  y = screen_height;
   }
 
   fprintf (stderr, "initial viewport %d %d\n", (int) screen_width,
@@ -466,13 +485,26 @@ main (int argc, char** argv)
 
   if (global_options.vertical_split || global_options.horizontal_split) 
   {
-      init_rect_bounds (viewport, 0, 0, fullwidth-x, fullheight-y);
+#ifdef HAVE_COMPOSITE
+	  if (!g_getenv ("MAGNIFIER_IGNORE_COMPOSITE"))
+		  init_rect_bounds (viewport, 0, 0, fullwidth, fullheight);
+	  else
+#endif /* HAVE_COMPOSITE */
+		  init_rect_bounds (viewport, 0, 0, fullwidth-x, fullheight-y);
       viewport_any = bonobo_arg_new_from (TC_GNOME_Magnifier_RectBounds, viewport);
       bonobo_pbclient_set_value (properties, "source-display-bounds",
 				 viewport_any,
 				 &ev);
 
       bonobo_arg_release (viewport_any);
+  } else if (global_options.fullscreen) {
+	  init_rect_bounds (viewport, 0, 0, fullwidth, fullheight);
+	  viewport_any = bonobo_arg_new_from (TC_GNOME_Magnifier_RectBounds,
+					      viewport);
+	  bonobo_pbclient_set_value (properties, "source-display-bounds",
+				     viewport_any,
+				     &ev);
+	  bonobo_arg_release (viewport_any);
   }
 
   bonobo_object_release_unref (properties, NULL);
@@ -562,11 +594,11 @@ main (int argc, char** argv)
   else
   {
 	  if (global_options.ignore_damage ||
-	      !magnifier_source_has_damage_extension (magnifier)) 
+	      !gmag_events_source_has_damage_extension (magnifier)) 
 	  {
-		  refresh_handle = g_timeout_add (
-			  global_options.refresh_time,
-			  magnifier_main_refresh_all, magnifier);
+	  	  refresh_handle = g_timeout_add (
+	  		  global_options.refresh_time,
+	  		  magnifier_main_refresh_all, magnifier);
 	  }
 	  
 	  pan_handle = g_timeout_add (
